@@ -27,6 +27,7 @@
 #include <bsp/irq.h>
 #include <mpc83xx/mpc83xx.h>
 #include <mpc83xx/tsec.h>
+#include <libcpu/spr.h>
 #include <rtems/error.h>
 #include <rtems/bspIo.h>
 #include <rtems/rtems_bsdnet.h>
@@ -43,6 +44,13 @@
 #include <netinet/if_ether.h>
 #include <stdio.h>
 
+/* System Version Register */
+#define SVR 286
+SPR_RO( SVR)
+
+/* Processor Version Register */
+SPR_RO( PVR)
+
 #define CLREVENT_IN_IRQ
 
 #define TSEC_WATCHDOG_TIMEOUT 5 /* check media every 5 seconds */
@@ -57,12 +65,13 @@ struct mpc83xx_tsec_struct {
   /*
    * HW links: (filled from rtems_bsdnet_ifconfig
    */
-  m83xxTSEC_Registers_t  *reg_ptr;     /* pointer to TSEC register block */
+  m83xxTSEC_Registers_t  *reg_ptr;    /* pointer to TSEC register block */
+  m83xxTSEC_Registers_t  *mdio_ptr;   /* pointer to TSEC register block which is responsible for MDIO communication */
   int                    irq_num_tx;  /* tx irq number                  */
   int                    irq_num_rx;  /* rx irq number                  */
   int                    irq_num_err; /* error irq number               */
-  
-  /* 
+
+  /*
    * BD management
    */
   int                     rxBdCount;
@@ -78,10 +87,10 @@ struct mpc83xx_tsec_struct {
   PQBufferDescriptor_t    *Tx_NxtUsed_BD; /* First BD, which is in Use */
   PQBufferDescriptor_t    *Tx_NxtFill_BD; /* BD to be filled next      */
   struct mbuf             **Tx_mBuf_Ptr;  /* Storage for mbufs         */
-  /* 
-   * Daemon IDs 
+  /*
+   * Daemon IDs
    */
-  rtems_id                rxDaemonTid; 
+  rtems_id                rxDaemonTid;
   rtems_id                txDaemonTid;
 
   /*
@@ -94,23 +103,12 @@ struct mpc83xx_tsec_struct {
    * statistic counters Rx
    */
   unsigned long           rxInterrupts;
-  unsigned long           rxNotLast;
-  unsigned long           rxGiant;
-  unsigned long           rxNonOctet;
-  unsigned long           rxBadCRC;
-  unsigned long           rxOverrun;
-  unsigned long           rxCollision;
-
+  unsigned long           rxErrors;
   /*
    * statistic counters Tx
    */
   unsigned long           txInterrupts;
-  unsigned long           txDeferred;
-  unsigned long           txLateCollision;
-  unsigned long           txUnderrun;
-  unsigned long           txMisaligned;
-  unsigned long           rxNotFirst;
-  unsigned long           txRetryLimit;
+  unsigned long           txErrors;
   };
 
 static struct mpc83xx_tsec_struct tsec_driver[M83xx_TSEC_NIFACES];
@@ -126,14 +124,14 @@ static struct mpc83xx_tsec_struct tsec_driver[M83xx_TSEC_NIFACES];
  */
 #define M83xx_IEVENT_TXALL (M83xx_TSEC_IEVENT_GTSC	\
 			    | M83xx_TSEC_IEVENT_TXC 	\
-			    | M83xx_TSEC_IEVENT_TXB 	\
+			    /*| M83xx_TSEC_IEVENT_TXB*/	\
 			    | M83xx_TSEC_IEVENT_TXF )
 
 /*
  * mask for all Rx interrupts
  */
 #define M83xx_IEVENT_RXALL (M83xx_TSEC_IEVENT_RXC  	\
-			    | M83xx_TSEC_IEVENT_RXB  	\
+			    /* | M83xx_TSEC_IEVENT_RXB */	\
 			    | M83xx_TSEC_IEVENT_GRSC 	\
 			    | M83xx_TSEC_IEVENT_RXF  )
 
@@ -203,7 +201,11 @@ static void mpc83xx_tsec_hwinit
 {
   m83xxTSEC_Registers_t  *reg_ptr = sc->reg_ptr; /* pointer to TSEC registers*/
   uint8_t *mac_addr;
-  int i;
+  size_t i;
+
+  /* Clear interrupt mask and all pending events */
+  reg_ptr->imask = 0;
+  reg_ptr->ievent = 0xffffffff;
 
   /*
    * init ECNTL register
@@ -212,8 +214,9 @@ static void mpc83xx_tsec_hwinit
    * NOTE: do not clear bits set in BSP init function
    */
   reg_ptr->ecntrl = ((reg_ptr->ecntrl & ~M83xx_TSEC_ECNTRL_AUTOZ)
-		     | M83xx_TSEC_ECNTRL_CLRCNT 
-		     | M83xx_TSEC_ECNTRL_STEN);
+		     | M83xx_TSEC_ECNTRL_CLRCNT
+		     | M83xx_TSEC_ECNTRL_STEN
+		     | M83xx_TSEC_ECNTRL_R100M);
 
   /*
    * init DMA control register:
@@ -234,14 +237,18 @@ static void mpc83xx_tsec_hwinit
 		   | M83xx_TSEC_ATTR_RBDSEN);
 
 
-  reg_ptr->mrblr  = MCLBYTES-64; /* take care of buffer size lost 
+  reg_ptr->mrblr  = MCLBYTES-64; /* take care of buffer size lost
 				  * due to alignment              */
 
   /*
-   * init EDIS register: enable all error reportings
-   * FIXME: make sure we handle these errors correctly
+   * init EDIS register: disable all error reportings
    */
-  reg_ptr->edis = 0;
+  reg_ptr->edis = (M83xx_TSEC_EDIS_BSYDIS    |
+		   M83xx_TSEC_EDIS_EBERRDIS  |
+		   M83xx_TSEC_EDIS_TXEDIS    |
+		   M83xx_TSEC_EDIS_LCDIS     |
+		   M83xx_TSEC_EDIS_CRLXDADIS |
+		   M83xx_TSEC_EDIS_FUNDIS);
   /*
    * init minimum frame length register
    */
@@ -258,25 +265,29 @@ static void mpc83xx_tsec_hwinit
    * init transmit interrupt coalescing register
    */
   reg_ptr->txic = (M83xx_TSEC_TXIC_ICEN
-		   | M83xx_TSEC_TXIC_ICFCT(16)
-		   | M83xx_TSEC_TXIC_ICTT(16));
+		   | M83xx_TSEC_TXIC_ICFCT(2)
+		   | M83xx_TSEC_TXIC_ICTT(32));
   /*
    * init receive interrupt coalescing register
    */
+#if 0
   reg_ptr->rxic = (M83xx_TSEC_RXIC_ICEN
-		   | M83xx_TSEC_RXIC_ICFCT(16)
-		   | M83xx_TSEC_RXIC_ICTT(16));
+		   | M83xx_TSEC_RXIC_ICFCT(2)
+		   | M83xx_TSEC_RXIC_ICTT(32));
+#else
+  reg_ptr->rxic = 0;
+#endif
   /*
    * init MACCFG1 register
    */
   reg_ptr->maccfg1 = (M83xx_TSEC_MACCFG1_RX_FLOW
 		      | M83xx_TSEC_MACCFG1_TX_FLOW);
-		      
+
   /*
    * init MACCFG2 register
    */
   reg_ptr->maccfg2 = ((reg_ptr->maccfg2 & M83xx_TSEC_MACCFG2_IFMODE_MSK)
-		      | M83xx_TSEC_MACCFG2_PRELEN(7)
+		      | M83xx_TSEC_MACCFG2_PRELEN( 7)
 		      | M83xx_TSEC_MACCFG2_FULLDUPLEX);
 
   /*
@@ -327,10 +338,19 @@ static void mpc83xx_tsec_mdio_init
 |    <none>                                                                 |
 \*=========================================================================*/
 {
+
+  /* Set TSEC registers for MDIO communication */
+
+  /*
+   * FIXME: Not clear if this works for all boards.
+   * Tested only on MPC8313ERDB.
+   */
+  sc->mdio_ptr = &mpc83xx.tsec [0];
+
   /*
    * set clock divider
    */
-  sc->reg_ptr->miimcfg = 3;
+  sc->mdio_ptr->miimcfg = 3;
 }
 
 /*=========================================================================*\
@@ -354,10 +374,10 @@ int mpc83xx_tsec_mdio_read
 |    0, if ok, else error                                                   |
 \*=========================================================================*/
 {
- struct mpc83xx_tsec_struct *sc = uarg;/* control structure                */
-  m83xxTSEC_Registers_t  *reg_ptr;     /* pointer to TSEC registers        */
+  struct mpc83xx_tsec_struct *sc = uarg;/* control structure                */
 
-  reg_ptr = sc->reg_ptr; 
+  /* pointer to TSEC registers */
+  m83xxTSEC_Registers_t *reg_ptr = sc->mdio_ptr;
 
   /*
    * make sure we work with a valid phy
@@ -375,9 +395,9 @@ int mpc83xx_tsec_mdio_read
     return EINVAL;
   }
   /*
-   * set PHY/reg address 
+   * set PHY/reg address
    */
-  reg_ptr->miimadd = (M83xx_TSEC_MIIMADD_PHY(phy) 
+  reg_ptr->miimadd = (M83xx_TSEC_MIIMADD_PHY(phy)
 		      | M83xx_TSEC_MIIMADD_REGADDR(reg));
   /*
    * start read cycle
@@ -422,10 +442,10 @@ int mpc83xx_tsec_mdio_write
 |    0, if ok, else error                                                   |
 \*=========================================================================*/
 {
- struct mpc83xx_tsec_struct *sc = uarg;/* control structure                */
-  m83xxTSEC_Registers_t  *reg_ptr;     /* pointer to TSEC registers        */
+  struct mpc83xx_tsec_struct *sc = uarg;/* control structure                */
 
-  reg_ptr = sc->reg_ptr; 
+  /* pointer to TSEC registers */
+  m83xxTSEC_Registers_t *reg_ptr = sc->mdio_ptr;
 
   /*
    * make sure we work with a valid phy
@@ -443,9 +463,9 @@ int mpc83xx_tsec_mdio_write
     return EINVAL;
   }
   /*
-   * set PHY/reg address 
+   * set PHY/reg address
    */
-  reg_ptr->miimadd = (M83xx_TSEC_MIIMADD_PHY(phy) 
+  reg_ptr->miimadd = (M83xx_TSEC_MIIMADD_PHY(phy)
 		      | M83xx_TSEC_MIIMADD_REGADDR(reg));
   /*
    * start write cycle
@@ -491,13 +511,13 @@ static rtems_event_set mpc83xx_tsec_rx_wait_for_events
   * enable Rx interrupts, make sure this is not interrupted :-)
   */
  M83xx_TSEC_IMASK_SET(sc->reg_ptr->imask,M83xx_IEVENT_RXALL,~0);
- 
+
  /*
   * wait for events to come in
   */
- rtems_bsdnet_event_receive(event_mask, 
-			    RTEMS_EVENT_ANY | RTEMS_WAIT, 
-			    RTEMS_NO_TIMEOUT, 
+ rtems_bsdnet_event_receive(event_mask,
+			    RTEMS_EVENT_ANY | RTEMS_WAIT,
+			    RTEMS_NO_TIMEOUT,
 			    &events);
  return events;
 }
@@ -538,7 +558,7 @@ static void mpc83xx_rxbd_alloc_clear
   sc->Rx_Frst_BD = (PQBufferDescriptor_t *)alloc_ptr;
   sc->Rx_NxtUsed_BD = sc->Rx_Frst_BD;
   sc->Rx_NxtFill_BD = sc->Rx_Frst_BD;
-  
+
   /*
    * clear all BDs
    */
@@ -573,7 +593,7 @@ static void mpc83xx_tsec_receive_packets
 {
   PQBufferDescriptor_t *BD_ptr;
   struct mbuf *m,*n;
-  boolean finished = FALSE;
+  bool finished = false;
   uint16_t status;
   struct ether_header *eh;
   int bd_idx;
@@ -582,7 +602,7 @@ static void mpc83xx_tsec_receive_packets
 
   while ((0 == ((status = BD_ptr->status) & M83xx_BD_EMPTY)) &&
 	 !finished &&
-	 (BD_ptr->buffer != NULL)) {    
+	 (BD_ptr->buffer != NULL)) {
     /*
      * get mbuf associated with BD
      */
@@ -604,8 +624,8 @@ static void mpc83xx_tsec_receive_packets
       /*
        * send mbuf of this buffer to ether_input()
        */
-      m->m_len   = m->m_pkthdr.len = (BD_ptr->length 
-				    - sizeof(uint32_t)  
+      m->m_len   = m->m_pkthdr.len = (BD_ptr->length
+				    - sizeof(uint32_t)
 				    - sizeof(struct ether_header));
       eh         = mtod(m, struct ether_header *);
       m->m_data += sizeof(struct ether_header);
@@ -616,26 +636,6 @@ static void mpc83xx_tsec_receive_packets
        * throw away mbuf
        */
       MFREE(m,n);
-      /*
-       * update statistics
-       */
-      if (0 != (status & M83xx_BD_LAST))
-        sc->rxNotLast++;
-      if (0 != (status & M83xx_BD_FIRST_IN_FRAME))
-        sc->rxNotFirst++;
-
-      if (0 == (status & M83xx_BD_LONG)) {
-        sc->rxGiant++;
-      }
-      if (0 == (status & M83xx_BD_NONALIGNED)) {
-        sc->rxNonOctet++;
-      }
-      if (0 == (status & M83xx_BD_CRC_ERROR)) {
-        sc->rxBadCRC++;
-      }
-      if (0 == (status & M83xx_BD_OVERRUN)) {
-        sc->rxOverrun++;
-      }
     }
     /*
      * mark buffer as non-allocated (for refill)
@@ -644,8 +644,8 @@ static void mpc83xx_tsec_receive_packets
     /*
      * Advance BD_ptr to next BD
      */
-    BD_ptr = ((BD_ptr == sc->Rx_Last_BD) 
-	     ? sc->Rx_Frst_BD 
+    BD_ptr = ((BD_ptr == sc->Rx_Last_BD)
+	     ? sc->Rx_Frst_BD
 	     : BD_ptr+1);
   }
   sc->Rx_NxtUsed_BD = BD_ptr;
@@ -671,11 +671,11 @@ static void mpc83xx_tsec_refill_rxbds
 {
   PQBufferDescriptor_t *BD_ptr;
   struct mbuf *m,*n;
-  boolean finished = FALSE;
+  bool finished = false;
   int bd_idx;
 
   BD_ptr = sc->Rx_NxtFill_BD;
-  while ((BD_ptr->buffer == NULL) && 
+  while ((BD_ptr->buffer == NULL) &&
 	 !finished) {
     /*
      * get new mbuf and attach a cluster
@@ -689,7 +689,7 @@ static void mpc83xx_tsec_refill_rxbds
       }
     }
     if (m == NULL) {
-      finished = TRUE;
+      finished = true;
     }
     else {
       bd_idx = BD_ptr - sc->Rx_Frst_BD;
@@ -699,16 +699,16 @@ static void mpc83xx_tsec_refill_rxbds
       m->m_data        = M83xx_TSEC_ALIGN_BUFFER(m->m_ext.ext_buf,64);
       BD_ptr->buffer   = m->m_data;
       BD_ptr->length   = 0;
-      BD_ptr->status   = (M83xx_BD_EMPTY 
-			  | M83xx_BD_INTERRUPT 
-			  | ((BD_ptr == sc->Rx_Last_BD) 
-			     ? M83xx_BD_WRAP 
+      BD_ptr->status   = (M83xx_BD_EMPTY
+			  | M83xx_BD_INTERRUPT
+			  | ((BD_ptr == sc->Rx_Last_BD)
+			     ? M83xx_BD_WRAP
 			     : 0));
       /*
        * Advance BD_ptr to next BD
        */
-      BD_ptr = ((BD_ptr == sc->Rx_Last_BD) 
-		? sc->Rx_Frst_BD 
+      BD_ptr = ((BD_ptr == sc->Rx_Last_BD)
+		? sc->Rx_Frst_BD
 		: BD_ptr+1);
     }
   }
@@ -733,9 +733,9 @@ static void mpc83xx_tsec_rxDaemon
 |    <none>                                                                 |
 \*=========================================================================*/
 {
-  struct mpc83xx_tsec_struct *sc = 
+  struct mpc83xx_tsec_struct *sc =
     (struct mpc83xx_tsec_struct *)arg;
-  int finished = FALSE;
+  bool finished = false;
   rtems_event_set events;
 #if !defined(CLREVENT_IN_IRQ)
   uint32_t irq_events;
@@ -817,7 +817,7 @@ static void mpc83xx_txbd_alloc_clear
   sc->Tx_Frst_BD = (PQBufferDescriptor_t *)alloc_ptr;
   sc->Tx_NxtUsed_BD = sc->Tx_Frst_BD;
   sc->Tx_NxtFill_BD = sc->Tx_Frst_BD;
-  
+
   /*
    * clear all BDs
    */
@@ -881,13 +881,13 @@ static rtems_event_set mpc83xx_tsec_tx_wait_for_events
   * enable Tx interrupts, make sure this is not interrupted :-)
   */
  M83xx_TSEC_IMASK_SET(sc->reg_ptr->imask,M83xx_IEVENT_TXALL,~0);
- 
+
  /*
   * wait for events to come in
   */
- rtems_bsdnet_event_receive(event_mask, 
-			    RTEMS_EVENT_ANY | RTEMS_WAIT, 
-			    RTEMS_NO_TIMEOUT, 
+ rtems_bsdnet_event_receive(event_mask,
+			    RTEMS_EVENT_ANY | RTEMS_WAIT,
+			    RTEMS_NO_TIMEOUT,
 			    &events);
  return events;
 }
@@ -929,8 +929,8 @@ static void mpc83xx_tsec_tx_retire
     /*
      * Advance CurrBD to next BD
      */
-    RetBD = ((RetBD == sc->Tx_Last_BD) 
-	     ? sc->Tx_Frst_BD 
+    RetBD = ((RetBD == sc->Tx_Last_BD)
+	     ? sc->Tx_Frst_BD
 	     : RetBD+1);
   }
   sc->Tx_NxtUsed_BD = RetBD;
@@ -971,7 +971,7 @@ static void mpc83xx_tsec_sendpacket
        */
       struct mbuf *n;
       MFREE(m, n);
-      m = n;      
+      m = n;
       if(l != NULL) {
         l->m_next = m;
       }
@@ -981,7 +981,7 @@ static void mpc83xx_tsec_sendpacket
        * this mbuf is non-empty, so send it
        */
       /*
-       * Is CurrBD still in Use/not yet retired? 
+       * Is CurrBD still in Use/not yet retired?
        */
       while (CurrBD->buffer != NULL) {
 	/*
@@ -997,11 +997,11 @@ static void mpc83xx_tsec_sendpacket
 	}
       }
       status = ((M83xx_BD_PAD_CRC | M83xx_BD_TX_CRC)
-		| ((m->m_next == NULL)        
-		   ? M83xx_BD_LAST | M83xx_BD_INTERRUPT 
+		| ((m->m_next == NULL)
+		   ? M83xx_BD_LAST | M83xx_BD_INTERRUPT
 		   : 0)
 		| ((CurrBD == sc->Tx_Last_BD) ? M83xx_BD_WRAP : 0));
-		
+
       /*
        * link buffer to BD
        */
@@ -1015,7 +1015,7 @@ static void mpc83xx_tsec_sendpacket
       m = m->m_next; /* advance to next mbuf of this packet */
       /*
        * is this the first BD of the packet?
-       * then don't set it to "READY" state, 
+       * then don't set it to "READY" state,
        * and remember this BD position
        */
       if (FrstBD == NULL) {
@@ -1028,14 +1028,14 @@ static void mpc83xx_tsec_sendpacket
       /*
        * Advance CurrBD to next BD
        */
-      CurrBD = ((CurrBD == sc->Tx_Last_BD) 
-		? sc->Tx_Frst_BD 
+      CurrBD = ((CurrBD == sc->Tx_Last_BD)
+		? sc->Tx_Frst_BD
 		: CurrBD+1);
     }
   }
   /*
-   * mbuf chain of this packet 
-   * has been translated 
+   * mbuf chain of this packet
+   * has been translated
    * to BD chain, so set first BD ready now
    */
   if (FrstBD != NULL) {
@@ -1066,11 +1066,11 @@ static void mpc83xx_tsec_txDaemon
 |    <none>                                                                 |
 \*=========================================================================*/
 {
-  struct mpc83xx_tsec_struct *sc = 
+  struct mpc83xx_tsec_struct *sc =
     (struct mpc83xx_tsec_struct *)arg;
   struct ifnet *ifp = &sc->arpcom.ac_if;
   struct mbuf *m;
-  int finished = FALSE;
+  bool finished = false;
   rtems_event_set events;
 #if !defined(CLREVENT_IN_IRQ)
   uint32_t irq_events;
@@ -1086,7 +1086,7 @@ static void mpc83xx_tsec_txDaemon
      * wait for events to come in
      */
     events = mpc83xx_tsec_tx_wait_for_events(sc,
-					     START_TRANSMIT_EVENT 
+					     START_TRANSMIT_EVENT
 					     | INTERRUPT_EVENT);
 #if !defined(CLREVENT_IN_IRQ)
     /*
@@ -1107,13 +1107,13 @@ static void mpc83xx_tsec_txDaemon
        * Get the next mbuf chain to transmit.
        */
       IF_DEQUEUE(&ifp->if_snd, m);
-      
+
       if (m) {
 	mpc83xx_tsec_sendpacket(sc,m);
       }
     } while (m != NULL);
 
-    ifp->if_flags &= ~IFF_OACTIVE; 
+    ifp->if_flags &= ~IFF_OACTIVE;
   }
   /*
    * disable Tx in MACCFG1 register
@@ -1148,7 +1148,7 @@ static void mpc83xx_tsec_tx_irq_handler
 |    <none>                                                                 |
 \*=========================================================================*/
 {
-  struct mpc83xx_tsec_struct *sc = 
+  struct mpc83xx_tsec_struct *sc =
     (struct mpc83xx_tsec_struct *)handle;
 #if defined(CLREVENT_IN_IRQ)
   uint32_t irq_events;
@@ -1191,7 +1191,7 @@ static void mpc83xx_tsec_rx_irq_handler
 |    <none>                                                                 |
 \*=========================================================================*/
 {
-  struct mpc83xx_tsec_struct *sc = 
+  struct mpc83xx_tsec_struct *sc =
     (struct mpc83xx_tsec_struct *)handle;
 #if defined(CLREVENT_IN_IRQ)
   uint32_t irq_events;
@@ -1234,18 +1234,26 @@ static void mpc83xx_tsec_err_irq_handler
 |    <none>                                                                 |
 \*=========================================================================*/
 {
+  struct mpc83xx_tsec_struct *sc =
+    (struct mpc83xx_tsec_struct *)handle;
   /*
-   * FIXME: check error conditions, do something useful
+   * clear error events in IEVENT
    */
-#if 0
+  sc->reg_ptr->ievent = M83xx_IEVENT_ERRALL;
   /*
-   * disable error interrupts
+   * has Rx been stopped? then restart it
    */
-  M83xx_TSEC_IMASK_SET(sc->reg_ptr->imask,M83xx_IEVENT_ERRALL,0);
+  if (0 != (sc->reg_ptr->rstat & M83xx_TSEC_RSTAT_QHLT)) {
+    sc->rxErrors++;
+    sc->reg_ptr->rstat = M83xx_TSEC_RSTAT_QHLT;
+  }
   /*
-   * FIXME: do something :-)
+   * has Tx been stopped? then restart it
    */
-#endif
+  if (0 != (sc->reg_ptr->tstat & M83xx_TSEC_TSTAT_THLT)) {
+    sc->txErrors++;
+    sc->reg_ptr->tstat = M83xx_TSEC_TSTAT_THLT;
+  }
 }
 
 
@@ -1259,9 +1267,9 @@ static uint32_t mpc83xx_tsec_irq_mask
 |   determine irq mask for given interrupt number                           |
 +---------------------------------------------------------------------------+
 | Input Parameters:                                                         |
-\*-------------------------------------------------------------------------*/ 
+\*-------------------------------------------------------------------------*/
  int irqnum,
- struct mpc83xx_tsec_struct *sc 
+ struct mpc83xx_tsec_struct *sc
 )
 /*-------------------------------------------------------------------------*\
 | Return Value:                                                             |
@@ -1295,7 +1303,7 @@ static void mpc83xx_tsec_irq_on
 |    <none>                                                                 |
 \*=========================================================================*/
 {
-  struct mpc83xx_tsec_struct *sc = 
+  struct mpc83xx_tsec_struct *sc =
     (struct mpc83xx_tsec_struct *)(irq_conn_data->handle);
 
   M83xx_TSEC_IMASK_SET(sc->reg_ptr->imask,
@@ -1322,7 +1330,7 @@ static void mpc83xx_tsec_irq_off
 |    <none>                                                                 |
 \*=========================================================================*/
 {
-  struct mpc83xx_tsec_struct *sc = 
+  struct mpc83xx_tsec_struct *sc =
     (struct mpc83xx_tsec_struct *)irq_conn_data->handle;
 
   M83xx_TSEC_IMASK_SET(sc->reg_ptr->imask,
@@ -1349,10 +1357,10 @@ static int mpc83xx_tsec_irq_isOn
 |    <none>                                                                 |
 \*=========================================================================*/
 {
-  struct mpc83xx_tsec_struct *sc = 
+  struct mpc83xx_tsec_struct *sc =
     (struct mpc83xx_tsec_struct *)irq_conn_data->handle;
 
-  return (0 != (sc->reg_ptr->imask 
+  return (0 != (sc->reg_ptr->imask
 		& mpc83xx_tsec_irq_mask(irq_conn_data->name,sc)));
 }
 
@@ -1368,14 +1376,14 @@ static void mpc83xx_tsec_install_irq_handlers
 | Input Parameters:                                                         |
 \*-------------------------------------------------------------------------*/
  struct mpc83xx_tsec_struct *sc,        /* ptr to control structure        */
- int install                            /* TRUE: install, FALSE: remove    */
+ bool   install                         /* true: install, false: remove    */
 )
 /*-------------------------------------------------------------------------*\
 | Return Value:                                                             |
 |    <none>                                                                 |
 \*=========================================================================*/
 {
-  int i;
+  size_t i;
 
   rtems_irq_connect_data irq_conn_data[3] = {
     {
@@ -1458,11 +1466,11 @@ static void mpc83xx_tsec_init
      * allocate storage for mbuf ptrs
      */
     sc->Rx_mBuf_Ptr = calloc(sc->rxBdCount,sizeof(struct mbuf *));
-    sc->Tx_mBuf_Ptr = calloc(sc->txBdCount,sizeof(struct mbuf *));    
+    sc->Tx_mBuf_Ptr = calloc(sc->txBdCount,sizeof(struct mbuf *));
     if ((sc->Rx_mBuf_Ptr == NULL) ||
 	(sc->Tx_mBuf_Ptr == NULL)) {
 	rtems_panic("TSEC: cannot allocate buffers for mbuf management");
-      
+
     }
 
     /*
@@ -1478,17 +1486,17 @@ static void mpc83xx_tsec_init
     /*
      * Start driver tasks
      */
-    sc->txDaemonTid = rtems_bsdnet_newproc("TStx", 
-					   4096, 
-					   mpc83xx_tsec_txDaemon, 
+    sc->txDaemonTid = rtems_bsdnet_newproc("TStx",
+					   4096,
+					   mpc83xx_tsec_txDaemon,
 					   sc);
-    sc->rxDaemonTid = rtems_bsdnet_newproc("TSrx", 4096, 
-					   mpc83xx_tsec_rxDaemon, 
+    sc->rxDaemonTid = rtems_bsdnet_newproc("TSrx", 4096,
+					   mpc83xx_tsec_rxDaemon,
 					   sc);
     /*
      * install interrupt handlers
      */
-    mpc83xx_tsec_install_irq_handlers(sc,1);
+    mpc83xx_tsec_install_irq_handlers(sc,true);
   }
   /*
    * Set flags appropriately
@@ -1500,6 +1508,14 @@ static void mpc83xx_tsec_init
     sc->reg_ptr->rctrl &= ~M83xx_TSEC_RCTRL_PROM;
   }
 
+#if defined(HSC_CM01)
+  /*
+   * for HSC CM01: we need to configure the PHY to use maximum skew adjust
+   */
+
+  mpc83xx_tsec_mdio_write(-1,sc,23,0x0100);
+#endif
+
   /*
    * init timer so the "watchdog function gets called periodically
    */
@@ -1529,7 +1545,7 @@ static void mpc83xx_tsec_off
 \*=========================================================================*/
 {
   /*
-   * FIXME: deinitialize driver
+   * deinitialize driver?
    */
 }
 
@@ -1540,7 +1556,7 @@ static void mpc83xx_tsec_stats
 (
 /*-------------------------------------------------------------------------*\
 | Purpose:                                                                  |
-|   perform io control functions                                            |
+|   print statistics                                                        |
 +---------------------------------------------------------------------------+
 | Input Parameters:                                                         |
 \*-------------------------------------------------------------------------*/
@@ -1565,24 +1581,50 @@ static void mpc83xx_tsec_stats
     rtems_ifmedia2str(media,NULL,0);
     printf ("\n");
   }
+#if 0 /* print all PHY registers */
+  {
+    int reg;
+    uint32_t reg_val;
+    printf("****** PHY register values****\n");
+    for (reg = 0;reg <= 31;reg++) {
+      mpc83xx_tsec_mdio_read(-1,sc,reg,&reg_val);
+      printf("%02d:0x%04x%c",reg,reg_val,
+	     (((reg % 4) == 3) ? '\n' : ' '));
+    }
+  }
+#endif
   /*
    * print some statistics
    */
-  printf ("  Rx Interrupts:%-8lu",   sc->rxInterrupts);
-  printf ("      Not First:%-8lu",   sc->rxNotFirst);
-  printf ("       Not Last:%-8lu\n", sc->rxNotLast);
-  printf ("          Giant:%-8lu",   sc->rxGiant);
-  printf ("      Non-octet:%-8lu\n", sc->rxNonOctet);
-  printf ("        Bad CRC:%-8lu",   sc->rxBadCRC);
-  printf ("        Overrun:%-8lu",   sc->rxOverrun);
-  printf ("      Collision:%-8lu\n", sc->rxCollision);
-  
-  printf ("  Tx Interrupts:%-8lu",   sc->txInterrupts);
-  printf ("       Deferred:%-8lu",   sc->txDeferred);
-  printf (" Late Collision:%-8lu\n", sc->txLateCollision);
-  printf ("Retransmit Limit:%-8lu",   sc->txRetryLimit);
-  printf ("        Underrun:%-8lu",   sc->txUnderrun);
-  printf ("     Misaligned:%-8lu\n", sc->txMisaligned);
+  printf ("   Rx Interrupts:%-8lu",   sc->rxInterrupts);
+  printf ("       Rx Errors:%-8lu",   sc->rxErrors);
+  printf ("      Rx packets:%-8lu\n",
+	  sc->reg_ptr->rmon_mib[m83xx_tsec_rmon_rpkt]);
+  printf ("   Rx broadcasts:%-8lu",
+	  sc->reg_ptr->rmon_mib[m83xx_tsec_rmon_rbca]);
+  printf ("   Rx multicasts:%-8lu",
+	  sc->reg_ptr->rmon_mib[m83xx_tsec_rmon_rmca]);
+  printf ("           Giant:%-8lu\n",
+	  sc->reg_ptr->rmon_mib[m83xx_tsec_rmon_rovr]);
+  printf ("       Non-octet:%-8lu",
+	  sc->reg_ptr->rmon_mib[m83xx_tsec_rmon_raln]);
+  printf ("         Bad CRC:%-8lu",
+	  sc->reg_ptr->rmon_mib[m83xx_tsec_rmon_rfcs]);
+  printf ("         Overrun:%-8lu\n",
+	  sc->reg_ptr->rmon_mib[m83xx_tsec_rmon_rdrp]);
+
+  printf ("   Tx Interrupts:%-8lu",   sc->txInterrupts);
+  printf ("       Tx Errors:%-8lu",   sc->txErrors);
+  printf ("      Tx packets:%-8lu\n",
+	  sc->reg_ptr->rmon_mib[m83xx_tsec_rmon_tpkt]);
+  printf ("        Deferred:%-8lu",
+	  sc->reg_ptr->rmon_mib[m83xx_tsec_rmon_tdfr]);
+  printf ("  Late Collision:%-8lu",
+	  sc->reg_ptr->rmon_mib[m83xx_tsec_rmon_tlcl]);
+  printf ("Retransmit Limit:%-8lu\n",
+	  sc->reg_ptr->rmon_mib[m83xx_tsec_rmon_tedf]);
+  printf ("        Underrun:%-8lu\n",
+	  sc->reg_ptr->rmon_mib[m83xx_tsec_rmon_tund]);
 }
 
 /*=========================================================================*\
@@ -1632,7 +1674,7 @@ static int mpc83xx_tsec_ioctl
       mpc83xx_tsec_off(sc);
     }
     if (ifp->if_flags & IFF_UP) {
-      mpc83xx_tsec_off(sc);
+      mpc83xx_tsec_init(sc);
     }
     break;
 
@@ -1644,7 +1686,7 @@ static int mpc83xx_tsec_ioctl
     break;
 
     /*
-     * FIXME: All sorts of multicast commands need to be added here!
+     * All sorts of multicast commands need to be added here!
      */
   default:
     error = EINVAL;
@@ -1674,9 +1716,9 @@ int rtems_mpc83xx_tsec_mode_adapt
 |    0, if success                                                       |
 \*=========================================================================*/
 {
-  int result;
-  int media;
+  int result = 0;
   struct mpc83xx_tsec_struct *sc = ifp->if_softc;
+  int media = IFM_MAKEWORD( 0, 0, 0, sc->phy_default);
 
 #ifdef DEBUG
   printf("c");
@@ -1718,13 +1760,6 @@ int rtems_mpc83xx_tsec_mode_adapt
       if (result != 0) {
 	return result;
       }
-#ifdef DEBUG
-      /*
-       * test: print current status
-       */
-      rtems_ifmedia2str(media,NULL,0);
-      printf ("\n");
-#endif
     } while (IFM_NONE == IFM_SUBTYPE(media));
   }
 
@@ -1732,18 +1767,27 @@ int rtems_mpc83xx_tsec_mode_adapt
    * now set HW according to media results:
    */
   /*
-   * if we are 1000MBit, then switch IF to GMII/byte mode
+   * if we are 1000MBit, then switch IF to byte mode
    */
   if (IFM_1000_T == IFM_SUBTYPE(media)) {
-    sc->reg_ptr->maccfg2 = 
+    sc->reg_ptr->maccfg2 =
       ((sc->reg_ptr->maccfg2 & ~M83xx_TSEC_MACCFG2_IFMODE_MSK)
        | M83xx_TSEC_MACCFG2_IFMODE_BYT);
   }
   else {
-    sc->reg_ptr->maccfg2 = 
+    sc->reg_ptr->maccfg2 =
       ((sc->reg_ptr->maccfg2 & ~M83xx_TSEC_MACCFG2_IFMODE_MSK)
        | M83xx_TSEC_MACCFG2_IFMODE_NIB);
   }
+  /*
+   * if we are 10MBit, then switch rate to 10M
+   */
+  if (IFM_10_T == IFM_SUBTYPE(media)) {
+    sc->reg_ptr->ecntrl &= ~M83xx_TSEC_ECNTRL_R100M;
+  }
+  else {
+    sc->reg_ptr->ecntrl |= M83xx_TSEC_ECNTRL_R100M;
+  }
   /*
    * if we are half duplex then switch to half duplex
    */
@@ -1752,7 +1796,7 @@ int rtems_mpc83xx_tsec_mode_adapt
   }
   else {
     sc->reg_ptr->maccfg2 |=  M83xx_TSEC_MACCFG2_FULLDUPLEX;
-  }    
+  }
   /*
    * store current media state for future compares
    */
@@ -1781,7 +1825,7 @@ static void mpc83xx_tsec_watchdog
 \*=========================================================================*/
 {
   rtems_mpc83xx_tsec_mode_adapt(ifp);
-  ifp->if_timer    = TSEC_WATCHDOG_TIMEOUT; 
+  ifp->if_timer    = TSEC_WATCHDOG_TIMEOUT;
 }
 
 /*=========================================================================*\
@@ -1799,13 +1843,15 @@ static int mpc83xx_tsec_driver_attach
 )
 /*-------------------------------------------------------------------------*\
 | Return Value:                                                             |
-|    zero, if success                                                       |
+|    1, if success                                                          |
 \*=========================================================================*/
 {
   struct mpc83xx_tsec_struct *sc;
   struct ifnet *ifp;
   int    unitNumber;
   char   *unitName;
+  uint32_t svr = _read_SVR();
+  uint32_t pvr = _read_PVR();
 
  /*
   * Parse driver name
@@ -1852,10 +1898,30 @@ static int mpc83xx_tsec_driver_attach
 
   /* get pointer to TSEC register block */
   sc->reg_ptr         = &mpc83xx.tsec[unitNumber-1];
-  /* get base interrupt number (for Tx irq, Rx=base+1,Err=base+2) */
-  sc->irq_num_tx      = config->irno + 0;  /* tx  irq number from BSP */
-  sc->irq_num_rx      = config->irno + 1;  /* rx  irq number from BSP */
-  sc->irq_num_err     = config->irno + 2;  /* err irq number from BSP */
+
+  if (svr == 0x80b00010 && pvr == 0x80850010) {
+    /*
+     * This is a special case for MPC8313ERDB with silicon revision 1.  Look in
+     * "MPC8313ECE Rev. 3, 3/2008" errata for "IPIC 1".
+     */
+    if (unitNumber == 1) {
+      sc->irq_num_tx      = 37;
+      sc->irq_num_rx      = 36;
+      sc->irq_num_err     = 35;
+    } else if (unitNumber == 2) {
+      sc->irq_num_tx      = 34;
+      sc->irq_num_rx      = 33;
+      sc->irq_num_err     = 32;
+    } else {
+      return 0;
+    }
+  } else {
+    /* get base interrupt number (for Tx irq, Rx=base+1,Err=base+2) */
+    sc->irq_num_tx      = config->irno + 0;  /* tx  irq number from BSP */
+    sc->irq_num_rx      = config->irno + 1;  /* rx  irq number from BSP */
+    sc->irq_num_err     = config->irno + 2;  /* err irq number from BSP */
+  }
+
   if (config->irno  == 0) {
     rtems_panic("TSEC: interupt base number irno not defined");
   }
@@ -1865,11 +1931,22 @@ static int mpc83xx_tsec_driver_attach
   sc->mdio_info.mdio_r   = mpc83xx_tsec_mdio_read;
   sc->mdio_info.mdio_w   = mpc83xx_tsec_mdio_write;
   sc->mdio_info.has_gmii = 1; /* we support gigabit IF */
+
   /*
    * XXX: Although most hardware builders will assign the PHY addresses
    * like this, this should be more configurable
    */
-  sc->phy_default = unitNumber-1; 
+#ifdef MPC8313ERDB
+  if (unitNumber == 2) {
+	  sc->phy_default = 4;
+  } else {
+	  /* TODO */
+	  return 0;
+  }
+#else /* MPC8313ERDB */
+  sc->phy_default = unitNumber-1;
+#endif /* MPC8313ERDB */
+
  /*
   * Set up network interface values
   */
@@ -1900,7 +1977,6 @@ static int mpc83xx_tsec_driver_attach
   return 1;
 }
 
-
 /*=========================================================================*\
 | Function:                                                                 |
 \*-------------------------------------------------------------------------*/
