@@ -2,14 +2,14 @@
  *  Rate Monotonic Manager
  *
  *
- *  COPYRIGHT (c) 1989-1999.
+ *  COPYRIGHT (c) 1989-2008.
  *  On-Line Applications Research Corporation (OAR).
  *
  *  The license and distribution terms for this file may be
  *  found in the file LICENSE in this distribution or at
  *  http://www.rtems.com/license/LICENSE.
  *
- *  $Id: no-rtmon.c,v 1.13 2007/05/09 18:44:20 joel Exp $
+ *  $Id$
  */
 
 #include <rtems/system.h>
@@ -22,8 +22,6 @@
 #include <rtems/rtems/types.h>
 #include <rtems/rtems/ratemon.h>
 
-void _Rate_monotonic_Manager_initialization(
-  uint32_t   maximum_periods
-)
+void _Rate_monotonic_Manager_initialization(void)
 {
 }
