@@ -17,7 +17,7 @@
  * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
-#define RCSID	"$Id: rtemsmain.c,v 1.8 2007/03/27 15:47:35 ralf Exp $"
+#define RCSID	"$Id$"
 
 #include <stdio.h>
 #include <ctype.h>
@@ -160,9 +160,9 @@ struct protent *protocols[] = {
 };
 
 int
-pppdmain(argc, argv)
-    int argc;
-    char *argv[];
+pppdmain(
+    int argc,
+    char *argv[])
 {
     int i, fdflags, t;
     char *connector;
@@ -478,8 +478,8 @@ pppdmain(argc, argv)
  * holdoff_end - called via a timeout when the holdoff period ends.
  */
 static void
-holdoff_end(arg)
-    void *arg;
+holdoff_end(
+    void *arg)
 {
     new_phase(PHASE_DORMANT);
 }
@@ -557,8 +557,8 @@ struct protocol_list {
  * protocol_name - find a name for a PPP protocol.
  */
 const char *
-protocol_name(proto)
-    int proto;
+protocol_name(
+    int proto)
 {
     struct protocol_list *lp;
 
@@ -657,8 +657,8 @@ get_input(void)
  * new_phase - signal the start of a new phase of pppd's operation.
  */
 void
-new_phase(p)
-    int p;
+new_phase(
+    int p)
 {
     pppd_phase = p;
     if (new_phase_hook)
@@ -669,8 +669,8 @@ new_phase(p)
  * die - clean up state and exit with the specified status.
  */
 void
-die(status)
-    int status;
+die(
+    int status)
 {
     cleanup();
 }
@@ -680,7 +680,7 @@ die(status)
  */
 /* ARGSUSED */
 static void
-cleanup()
+cleanup(void)
 {
     sys_cleanup();
 
@@ -696,7 +696,7 @@ cleanup()
  * close_tty - restore the terminal device and close it.
  */
 static void
-close_tty()
+close_tty(void)
 {
     /* drop dtr to hang up */
     if (!default_device && modem) {
@@ -718,8 +718,8 @@ close_tty()
  * update_link_stats - get stats at link termination.
  */
 void
-update_link_stats(u)
-    int u;
+update_link_stats(
+    int u)
 {
     struct timeval now;
     char numbuf[32];
@@ -752,15 +752,15 @@ static struct timeval timenow;		/* Current time */
  * the kernel).
  */
 void
-ppptimeout(func, arg, time)
-    void (*func)(void *);
-    void *arg;
-    int time;
+ppptimeout(
+    void (*func)(void *),
+    void *arg,
+    int time)
 {
     struct callout *newp, *p, **pp;
-  
+
     MAINDEBUG(("Timeout %p:%p in %d seconds.", func, arg, time));
-  
+
     /*
      * Allocate timeout.
      */
@@ -771,7 +771,7 @@ ppptimeout(func, arg, time)
     gettimeofday(&timenow, NULL);
     newp->c_time.tv_sec = timenow.tv_sec + time;
     newp->c_time.tv_usec = timenow.tv_usec;
-  
+
     /*
      * Find correct place and link it in.
      */
@@ -789,14 +789,14 @@ ppptimeout(func, arg, time)
  * untimeout - Unschedule a timeout.
  */
 void
-pppuntimeout(func, arg)
-    void (*func)(void *);
-    void *arg;
+pppuntimeout(
+    void (*func)(void *),
+    void *arg)
 {
     struct callout **copp, *freep;
-  
+
     MAINDEBUG(("Untimeout %p:%p.", func, arg));
-  
+
     /*
      * Find first matching timeout and remove it from the list.
      */
@@ -813,7 +813,7 @@ pppuntimeout(func, arg)
  * calltimeout - Call any timeout routines which are now due.
  */
 static void
-calltimeout()
+calltimeout(void)
 {
     struct callout *p;
 
@@ -839,8 +839,8 @@ calltimeout()
  * timeleft - return the length of time until the next timeout is due.
  */
 static struct timeval *
-timeleft(tvp)
-    struct timeval *tvp;
+timeleft(
+    struct timeval *tvp)
 {
     if (callout == NULL)
 	return NULL;
@@ -889,8 +889,8 @@ static int device_script(int fd, int mode, char *program)
  * novm - log an error message saying we ran out of memory, and die.
  */
 void
-novm(msg)
-    char *msg;
+novm(
+    char *msg)
 {
     fatal("Virtual memory exhausted allocating %s\n", msg);
 }
