@@ -1,15 +1,20 @@
+/**
+ * @file
+ *
+ * @ingroup ClassicUserExtensions
+ *
+ * @brief User Extensions Implementation.
+ */
+
 /*
- *  Extension Manager -- rtems_extension_create
- *
- *
- *  COPYRIGHT (c) 1989-2002.
+ *  COPYRIGHT (c) 1989-2007.
  *  On-Line Applications Research Corporation (OAR).
  *
  *  The license and distribution terms for this file may be
  *  found in the file LICENSE in this distribution or at
  *  http://www.rtems.com/license/LICENSE.
  *
- *  $Id: extensioncreate.c,v 1.4 2005/06/09 14:13:35 joel Exp $
+ *  $Id$
  */
 
 #if HAVE_CONFIG_H
@@ -22,31 +27,17 @@
 #include <rtems/score/thread.h>
 #include <rtems/extension.h>
 
-/*PAGE
- *
- *  rtems_extension_create
- *
- *  This directive creates a extension and performs some initialization.
- *
- *  Input parameters:
- *    name            - extension name
- *    extension_table - pointer to extension set information
- *    id              - pointer to extension id
- *
- *  Output parameters:
- *    id                - extension id
- *    RTEMS_SUCCESSFUL - if successful
- *    error code        - if unsuccessful
- */
-
 rtems_status_code rtems_extension_create(
-  rtems_name              name,
-  rtems_extensions_table *extension_table,
-  Objects_Id             *id
+  rtems_name                    name,
+  const rtems_extensions_table *extension_table,
+  rtems_id                     *id
 )
 {
   Extension_Control *the_extension;
 
+  if ( !id )
+    return RTEMS_INVALID_ADDRESS;
+
   if ( !rtems_is_name_valid( name ) )
     return RTEMS_INVALID_NAME;
 
@@ -59,7 +50,7 @@ rtems_status_code rtems_extension_create(
     return RTEMS_TOO_MANY;
   }
 
-  _User_extensions_Add_set( &the_extension->Extension, extension_table );
+  _User_extensions_Add_set_with_table( &the_extension->Extension, extension_table );
 
   _Objects_Open(
     &_Extension_Information,
