@@ -14,7 +14,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * 
+ *
  *
  *  COPYRIGHT (c) 1998-2006.
  *  On-Line Applications Research Corporation (OAR).
@@ -23,7 +23,7 @@
  *  found in the file LICENSE in this distribution or at
  *  http://www.rtems.com/license/LICENSE.
  *
- *  $Id: cpu.h,v 1.21 2007/05/09 15:28:46 ralf Exp $
+ *  $Id$
  */
 
 #ifndef _RTEMS_SCORE_CPU_H
@@ -82,6 +82,19 @@ extern "C" {
 
 #define CPU_UNROLL_ENQUEUE_PRIORITY      TRUE
 
+/*
+ *  Does the CPU follow the simple vectored interrupt model?
+ *
+ *  If TRUE, then RTEMS allocates the vector table it internally manages.
+ *  If FALSE, then the BSP is assumed to allocate and manage the vector
+ *  table
+ *
+ *  SH Specific Information:
+ *
+ *  XXX document implementation including references if appropriate
+ */
+#define CPU_SIMPLE_VECTORED_INTERRUPTS TRUE
+
 /*
  *  Does RTEMS manage a dedicated interrupt stack in software?
  *
@@ -112,11 +125,11 @@ extern "C" {
 /*
  * We define the interrupt stack in the linker script
  */
-#define CPU_ALLOCATE_INTERRUPT_STACK FALSE 
+#define CPU_ALLOCATE_INTERRUPT_STACK FALSE
 
 /*
  *  Does the RTEMS invoke the user's ISR with the vector number and
- *  a pointer to the saved interrupt frame (1) or just the vector 
+ *  a pointer to the saved interrupt frame (1) or just the vector
  *  number (0)?
  */
 
@@ -280,7 +293,7 @@ extern "C" {
 #define CPU_BIG_ENDIAN                           TRUE
 #define CPU_LITTLE_ENDIAN                        FALSE
 #endif
- 
+
 /*
  *  The following defines the number of bits actually used in the
  *  interrupt field of the task mode.  How those bits map to the
@@ -361,10 +374,13 @@ typedef struct {
   uint32_t   *r0;	/* arg return */
 
   uint32_t   gbr;
-  uint32_t   sr; 
+  uint32_t   sr;
 
 } Context_Control;
 
+#define _CPU_Context_Get_SP( _context ) \
+  (_context)->r15
+
 typedef struct {
 #if SH_HAS_FPU
 #ifdef SH4_USE_X_REGISTERS
@@ -385,39 +401,6 @@ typedef struct {
 typedef struct {
 } CPU_Interrupt_frame;
 
-
-/*
- *  The following table contains the information required to configure
- *  the SH processor specific parameters.
- */
-
-typedef struct {
-  void       (*pretasking_hook)( void );
-  void       (*predriver_hook)( void );
-  void       (*postdriver_hook)( void );
-  void       (*idle_task)( void );
-  boolean      do_zero_of_workspace;
-  uint32_t     idle_task_stack_size;
-  uint32_t     interrupt_stack_size;
-  uint32_t     extra_mpci_receive_server_stack;
-  void *     (*stack_allocate_hook)( uint32_t   );
-  void       (*stack_free_hook)( void* );
-  /* end of fields required on all CPUs */
-  uint32_t  	clicks_per_second ; /* cpu frequency in Hz */
-}   rtems_cpu_table;
-
-/*
- *  Macros to access required entires in the CPU Table are in 
- *  the file rtems/system.h.
- */
-
-/*
- *  Macros to access SH specific additions to the CPU Table
- */
-
-#define rtems_cpu_configuration_get_clicks_per_second() \
-  (_CPU_Table.clicks_per_second)
-   
 /*
  *  This variable is optional.  It is used on CPUs on which it is difficult
  *  to generate an "uninitialized" FP context.  It is filled in by
@@ -445,18 +428,6 @@ SCORE_EXTERN Context_Control_fp  _CPU_Null_fp_context;
 SCORE_EXTERN void               *_CPU_Interrupt_stack_low;
 SCORE_EXTERN void               *_CPU_Interrupt_stack_high;
 
-/*
- *  With some compilation systems, it is difficult if not impossible to
- *  call a high-level language routine from assembly language.  This
- *  is especially true of commercial Ada compilers and name mangling
- *  C++ ones.  This variable can be optionally defined by the CPU porter
- *  and contains the address of the routine _Thread_Dispatch.  This
- *  can make it easier to invoke that routine at the end of the interrupt
- *  sequence (if a dispatch is necessary).
- */
-
-SCORE_EXTERN void           (*_CPU_Thread_dispatch_pointer)();
-
 /*
  *  Nothing prevents the porter from declaring more CPU specific variables.
  */
@@ -565,9 +536,9 @@ SCORE_EXTERN void CPU_delay( uint32_t   microseconds );
  *
  *  SH Specific Information: NONE
  */
- 
+
 #define _CPU_Initialize_vectors()
- 
+
 /*
  *  Disable all interrupts for an RTEMS critical section.  The previous
  *  level is returned in _level.
@@ -636,7 +607,7 @@ uint32_t   _CPU_ISR_Get_level( void );
  *        where the PSR contains an enable FPU bit.
  */
 
-/* 
+/*
  * FIXME: defined as a function for debugging - should be a macro
  */
 SCORE_EXTERN void _CPU_Context_Initialize(
@@ -842,18 +813,15 @@ extern uint8_t   _bit_set_table[];
  *  This routine performs CPU dependent initialization.
  */
 
-void _CPU_Initialize(
-  rtems_cpu_table  *cpu_table,
-  void      (*thread_dispatch)
-);
+void _CPU_Initialize(void);
 
 /*
  *  _CPU_ISR_install_raw_handler
  *
- *  This routine installs a "raw" interrupt handler directly into the 
+ *  This routine installs a "raw" interrupt handler directly into the
  *  processor's vector table.
  */
- 
+
 void _CPU_ISR_install_raw_handler(
   uint32_t    vector,
   proc_ptr    new_handler,
@@ -892,7 +860,7 @@ void _CPU_Install_interrupt_stack( void );
  *         is TRUE.
  */
 
-void _CPU_Thread_Idle_body( void );
+void *_CPU_Thread_Idle_body( uintptr_t ignored );
 
 /*
  *  _CPU_Context_switch
