@@ -1,4 +1,12 @@
 /**
+ * @file
+ *
+ * @ingroup ScoreProtHeap
+ *
+ * @brief Protected Heap Handler implementation.
+ */
+
+/*
  *  COPYRIGHT (c) 1989-2007.
  *  On-Line Applications Research Corporation (OAR).
  *
@@ -6,7 +14,7 @@
  *  found in the file LICENSE in this distribution or at
  *  http://www.rtems.com/license/LICENSE.
  *
- *  $Id: pheapgetfreeinfo.c,v 1.1 2007/05/03 21:33:39 joel Exp $
+ *  $Id$
  */
 
 #if HAVE_CONFIG_H
@@ -16,13 +24,19 @@
 #include <rtems/system.h>
 #include <rtems/score/protectedheap.h>
 
-void _Protected_heap_Get_free_information(
+bool _Protected_heap_Get_free_information(
   Heap_Control        *the_heap,
   Heap_Information    *info
 )
 {
+  /*
+   * TBD: _Heap_Get_free_information does not error check or return status.
+   */
+
   _RTEMS_Lock_allocator();
     _Heap_Get_free_information( the_heap, info );
   _RTEMS_Unlock_allocator();
+
+  return true;
 }
 
