@@ -1,17 +1,17 @@
 /*
  *  Simple test program -- simplified version of sample test hello.
  *
- *  COPYRIGHT (c) 1989-1999.
+ *  COPYRIGHT (c) 1989-2008.
  *  On-Line Applications Research Corporation (OAR).
  *
  *  The license and distribution terms for this file may be
  *  found in the file LICENSE in this distribution or at
  *  http://www.rtems.com/license/LICENSE.
  *
- *  $Id: main.c,v 1.2 2006/12/04 14:11:37 joel Exp $
+ *  $Id$
  */
 
-#define TEST_INIT
+#define CONFIGURE_INIT
 
 #include <bsp.h>
 #include <pmacros.h>
@@ -41,6 +41,8 @@ rtems_task Init(
 #define CONFIGURE_INIT_TASK_PRIORITY      2
 #define CONFIGURE_INIT_TASK_INITIAL_MODES RTEMS_PREEMPT
 
+#define CONFIGURE_EXTRA_TASK_STACKS       RTEMS_MINIMUM_STACK_SIZE
+
 #define CONFIGURE_INIT
 
 #include <rtems/confdefs.h>
